#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <lber.h>
#include <ldap.h>

#include "ldapconfig.h"

#define DEFSEP		"="

#ifdef LDAP_DEBUG
extern int ldap_debug, lber_debug;
#endif /* LDAP_DEBUG */

usage( s )
char	*s;
{
    fprintf( stderr, "usage: %s [-n] [-u] [-v] [-k] [-F sep] [-d debuglevel] [-f file]\n", s );
    fprintf( stderr, "    [-D binddn] [-w bindpasswd] [-h ldaphost] [-p ldapport] [-b searchbase ]\n" );
    fprintf( stderr, "    [-s scope ] [-a deref] filter [attrs....]\n" );
    exit( 1 );
}

static char	*binddn = LDAPSEARCH_BINDDN;
static char	*passwd = NULL;
static char	*base = LDAPSEARCH_BASE;
static char	*ldaphost = LDAPHOST;
static int	ldapport = LDAP_PORT;
static char	*sep = DEFSEP;
static int	verbose, not, includeufn;

main( argc, argv )
int	argc;
char	**argv;
{
    char		*infile, *filtpattern, **attrs, line[ BUFSIZ ];
    FILE		*fp;
    int			rc, i, first, scope, kerberos, deref;
    LDAP		*ld;
    extern char		*optarg;
    extern int		optind;

    infile = NULL;
    deref = verbose = not = kerberos = 0;
    scope = LDAP_SCOPE_SUBTREE;

    while (( i = getopt( argc, argv, "knuvD:s:f:h:b:d:p:F:a:w:" )) != EOF ) {
	switch( i ) {
	case 'n':	/* do Not do any searches */
	    ++not;
	    break;
	case 'v':	/* verbose mode */
	    ++verbose;
	    break;
	case 'd':
#ifdef LDAP_DEBUG
	    ldap_debug = lber_debug = atoi( optarg );	/* */
#else /* LDAP_DEBUG */
	    fprintf( stderr, "compile with -DLDAP_DEBUG for debugging\n" );
#endif /* LDAP_DEBUG */
	    break;
	case 'k':	/* use kerberos bind */
	    ++kerberos;
	    break;
	case 'u':	/* include UFN */
	    ++includeufn;
	    break;
	case 's':	/* search scope */
	    if ( strncasecmp( optarg, "base", 4 ) == 0 ) {
		scope = LDAP_SCOPE_BASE;
	    } else if ( strncasecmp( optarg, "one", 3 ) == 0 ) {
		scope = LDAP_SCOPE_ONELEVEL;
	    } else if ( strncasecmp( optarg, "sub", 3 ) == 0 ) {
		scope = LDAP_SCOPE_SUBTREE;
	    } else {
		fprintf( stderr, "scope should be base, one, or sub\n" );
		usage( argv[ 0 ] );
	    }
	    break;

	case 'a':	/* set alias deref option */
	    if ( strncasecmp( optarg, "never", 5 ) == 0 ) {
		deref = LDAP_DEREF_NEVER;
	    } else if ( strncasecmp( optarg, "search", 5 ) == 0 ) {
		deref = LDAP_DEREF_SEARCHING;
	    } else if ( strncasecmp( optarg, "find", 4 ) == 0 ) {
		deref = LDAP_DEREF_FINDING;
	    } else if ( strncasecmp( optarg, "always", 6 ) == 0 ) {
		deref = LDAP_DEREF_ALWAYS;
	    } else {
		fprintf( stderr, "alias deref should be never, search, find, or always\n" );
		usage( argv[ 0 ] );
	    }
	    break;
	    
	case 'F':	/* field separator */
	    sep = strdup( optarg );
	    break;
	case 'f':	/* input file */
	    infile = strdup( optarg );
	    break;
	case 'h':	/* ldap host */
	    ldaphost = strdup( optarg );
	    break;
	case 'b':	/* searchbase */
	    base = strdup( optarg );
	    break;
	case 'D':	/* bind DN */
	    binddn = strdup( optarg );
	    break;
	case 'p':	/* ldap port */
	    ldapport = atoi( optarg );
	    break;
	case 'w':	/* bind password */
	    passwd = strdup( optarg );
	    break;
	default:
	    usage( argv[0] );
	}
    }

    if ( argc - optind < 1 ) {
	usage( argv[ 0 ] );
    }
    filtpattern = argv[ optind++ ];
    attrs = &argv[ optind ];
    if ( attrs[ 0 ] == NULL ) {
	attrs = NULL;
    }

    if ( infile != NULL ) {
	if (( fp = fopen( infile, "r" )) == NULL ) {
	    perror( infile );
	    exit( 1 );
	}
    }

    if ( verbose ) {
	printf( "ldap_open( %s, %d )\n", ldaphost, ldapport );
    }

    if (( ld = ldap_open( ldaphost, ldapport )) == NULL ) {
	perror( ldaphost );
	exit( 1 );
    }

    ld->ld_deref = deref;

    if ( ldap_bind_s( ld, binddn, passwd,
	    kerberos ? LDAP_AUTH_KRBV4 : LDAP_AUTH_SIMPLE ) != LDAP_SUCCESS ) {
	ldap_perror( ld, "ldap_bind" );
	exit( 1 );
    }

    if ( verbose ) {
	printf( "filter pattern: %s\nreturning: ", filtpattern );
	if ( attrs == NULL ) {
	    printf( "ALL" );
	} else {
	    for ( i = 0; attrs[ i ] != NULL; ++i ) {
		printf( "%s ", attrs[ i ] );
	    }
	}
	putchar( '\n' );
    }

    if ( infile == NULL ) {
	rc = dosearch( ld, base, scope, attrs, filtpattern, "" );
    } else {
	rc = first = 0;
	while ( rc == 0 && fgets( line, sizeof( line ), fp ) != NULL ) {
	    line[ strlen( line ) - 1 ] = '\0';
	    if ( !first ) {
		putchar( '\n' );
	    } else {
		first = 0;
	    }
	    rc = dosearch( ld, base, scope, attrs, filtpattern, line );
	}
	fclose( fp );
    }

    ldap_unbind( ld );
    exit( rc );
}


dosearch( ld, base, scope, attrs, filtpatt, value )
    LDAP	*ld;
    char	*base;
    int		scope;
    char	**attrs;
    char	*filtpatt;
    char	*value;
{
    char		filter[ BUFSIZ ], **val;
    int			rc, first, matches;
    LDAPMessage		*res, *e;

    sprintf( filter, filtpatt, value );

    if ( verbose ) {
	printf( "filter is: (%s)\n", filter );
    }

    if ( not ) {
	return( LDAP_SUCCESS );
    }

    if ( ldap_search_s( ld, base, scope, filter, attrs, 0, &res ) == -1 ) {
	ldap_perror( ld, "ldap_search" );
	return( ld->ld_errno );
    }

    if (( rc = ld->ld_errno ) != LDAP_SUCCESS ) {
	ldap_perror( ld, "ldap_search" );
    }

    matches = ldap_count_entries( ld, res );

    if ( verbose ) {
	printf( "%d matches\n", matches );
    }

    first = 1;
    if ( matches > 0 ) {
	for ( e = ldap_first_entry( ld, res ); e != NULLMSG;
		e = ldap_next_entry( ld, e ) ) {
	    if ( !first ) {
		putchar( '\n' );
	    } else {
		first = 0;
	    }
	    print_entry( ld, e );
	}
    }

    ldap_msgfree( res );
    return( rc );
}


print_entry( ld, entry )
    LDAP	*ld;
    LDAPMessage	*entry;
{
    char		*p, *a, **vals, *dn, *ufn;
    int			i;
    BerElement		*ber;

    dn = ldap_get_dn( ld, entry );
    printf( "%s\n", dn );
    if ( includeufn ) {
	ufn = ldap_dn2ufn( dn );
	printf( "%s\n", ufn );
	free( ufn );
    }
    free( dn );

    for ( a = ldap_first_attribute( ld, entry, &ber ); a != NULL;
	    a = ldap_next_attribute( ld, entry, ber ) ) {
	if (( vals = ldap_get_values( ld, entry, a )) != NULL ) {
	    for ( i = 0; vals[i] != NULL; i++ ) {
		for ( p = vals[ i ]; *p != '\0'; ++p ) {
		    if ( !isascii( *p )) {
			break;
		    }
		}
		printf( "%s%s%s\n", a, sep,
			( *p == '\0' ) ? vals[ i ] : "NOT ASCII" );
	    }
	    ldap_value_free( vals );
	}
    }
}
